#! /usr/lib/veetest/veetest -r
(revision "B.00.00")
(date "Mon 26/Oct/1992")
(component 0 "ROOTCONTEXT"
 (name "To From File example")
 (interface
 )
 (implementation
  (locked no)
  (trigMode deg)
  (nextID 21)
  (component 0 "NOTE"
   (interface
   )
   (implementation
    (component text 3 5
     Press RUN to see
     the example operate
     or click Info About
    )
   )
   (views
    (icon
    )
    (detail
     (origin 347 103)
     (extent 187 74)
     (editing disabled)
    )
    (active detail)
   )
  )
  (component 1 "NOTE"
   (name "Warranty Disclaimer")
   (interface
   )
   (implementation
    (component text 4 5
     This example is provided as an
     illustration "as is", and Hewlett-Packard
     Company makes no warranty of any kind
     with regard to this example.
    )
   )
   (views
    (icon
     (origin 30 308)
     (extent 181 25)
    )
    (detail
     (origin 559 100)
     (extent 382 81)
     (editing disabled)
    )
    (active detail)
   )
  )
  (component 2 "DELAY"
   (name "Simulate Processing Delay")
   (interface
    (sequence in)
    (sequence out)
   )
   (implementation
    (component delay 1)
   )
   (views
    (icon
     (origin 71 678)
     (extent 235 25)
    )
    (detail
     (origin 36 836)
     (extent 309 28)
    )
    (active detail)
   )
  )
  (component 3 "COUNTER"
   (name "# Records Read")
   (interface
    (sequence in)
    (sequence out)
    (input 1
     (type data)
     (name "Data")
    )
    (output 1
     (type data)
     (name "Count")
     (tag "Count")
     (lock name constraints)
    )
   )
   (implementation
    (component clearAtPrerun 1)
    (component clearAtActivate 1)
   )
   (views
    (icon
     (extent 136 0)
    )
    (detail
     (origin 409 657)
     (extent 243 27)
    )
    (active detail)
   )
  )
  (component 4 "SHELLESCAPE"
   (name "rm /tmp/example.dat")
   (interface
    (sequence in)
    (sequence out)
    (output 1
     (type data)
     (name "Exit code")
     (lock name constraints)
    )
   )
   (implementation
    (attr iopath pipe read "pipe"
     (readTerm "\n")
     (fs " ")
     (eol "\n")
     (multiField fullSyntax)
     (arrayFormat block)
    )
    (attr iopath pipe write "pipe"
     (readTerm "\n")
     (fs " ")
     (eol "\n")
     (multiField fullSyntax)
     (arrayFormat block)
    )
    (component shell "sh")
    (component command "rm /tmp/example.data")
    (component deathWait "Yes")
   )
   (views
    (icon
     (origin 460 738)
     (extent 181 25)
    )
    (detail
     (origin 371 574)
     (extent 379 53)
    )
    (active icon)
   )
  )
  (component 5 "EXITPROCESS"
   (interface
    (sequence in)
   )
   (implementation
    (comp code 0)
   )
   (views
    (icon
     (origin 527 788)
     (extent 46 25)
    )
    (detail
     (origin 630 721)
     (extent 120 38)
    )
    (active icon)
   )
  )
  (component 6 "BREAK"
   (interface
    (sequence in)
   )
   (implementation
   )
   (views
    (icon
     (origin 301 628)
     (extent 78 25)
    )
    (detail
    )
    (terminals on)
    (active icon)
   )
  )
  (component 7 "NEXT"
   (interface
    (sequence in)
   )
   (implementation
   )
   (views
    (icon
     (origin 587 528)
     (extent 46 25)
    )
    (detail
    )
    (terminals on)
    (active icon)
   )
  )
  (component 8 "NEXT"
   (interface
    (sequence in)
   )
   (implementation
   )
   (views
    (icon
     (origin 587 588)
     (extent 46 25)
    )
    (detail
    )
    (terminals on)
    (active icon)
   )
  )
  (component 9 "FROMFILE"
   (description 15 4
    To allow this thread to wait until the data file
    exists and has at least one record of data, we
    need to loop repeatedly here.  
    
    The REWIND transaction will fail until the file
    exists.  Once the file exists, the READ TEXT will
    fail until at least one set of data is available
    in the file.  After both of these are satisfied,
    we need to REWIND the file so real data processing
    can begin.   
    
    Falling through to the Sequence Out pin and the
    BREAK will terminate this endless wait for data.
    
    
   )
   (interface
    (sequence in)
    (sequence out)
    (output 1
     (type data)
     (name "EOF")
     (tag "EOF")
     (lock name constraints)
     (optional yes)
    )
    (output 2
     (type exception)
     (name "Error")
     (lock name constraints)
     (optional yes)
    )
   )
   (implementation
    (attr iopath file read "/tmp/example.data"
     (readTerm "\n")
     (fs " ")
     (eol "\n")
     (multiField fullSyntax)
     (arrayFormat block)
    )
    (procedure
      EXECUTE REWIND
      READ TEXT null REAL COUNT:256
      EXECUTE REWIND
    )
   )
   (views
    (icon
     (origin 115 578)
     (extent 110 25)
    )
    (detail
     (origin 187 498)
     (extent 316 105)
    )
    (terminals on)
    (active detail)
   )
  )
  (component 10 "REPEATUNTILBREAK"
   (interface
    (sequence in)
    (sequence out)
    (output 1
     (type data)
     (name "Continuous")
     (lock name constraints)
    )
   )
   (implementation
   )
   (views
    (icon
     (origin 25 418)
     (extent 110 64)
     (iconImage "loop.icon")
    )
    (detail
    )
    (terminals on)
    (active icon)
   )
  )
  (component 11 "REPEATUNTILBREAK"
   (interface
    (sequence in)
    (sequence out)
    (output 1
     (type data)
     (name "Continuous")
     (lock name constraints)
    )
   )
   (implementation
   )
   (views
    (icon
     (origin 25 598)
     (extent 110 64)
     (iconImage "loop.icon")
    )
    (detail
    )
    (terminals on)
    (active icon)
   )
  )
  (component 12 "FROMFILE"
   (interface
    (sequence in)
    (sequence out)
    (output 1
     (type data)
     (name "X")
     (lock constraints)
     (optional yes)
    )
    (output 2
     (type data)
     (name "EOF")
     (tag "EOF")
     (lock name constraints)
     (optional yes)
    )
   )
   (implementation
    (attr iopath file read "/tmp/example.data"
     (readTerm "\n")
     (fs " ")
     (eol "\n")
     (multiField fullSyntax)
     (arrayFormat block)
    )
    (procedure
      READ TEXT x REAL COUNT:256
    )
   )
   (views
    (icon
     (origin 115 578)
     (extent 110 25)
    )
    (detail
     (origin 46 714)
     (extent 278 73)
    )
    (terminals on)
    (active detail)
   )
  )
  (component 13 "CONTEXT"
   (name "Press For")
   (interface
    (sequence in)
    (sequence out)
   )
   (implementation
    (locked no)
    (trigMode deg)
    (nextID 4)
    (component 0 "IFTHENELSE"
     (name "!= 0")
     (interface
      (sequence in)
      (sequence out)
      (input 1
       (type data)
       (name "A")
       (optional yes)
      )
      (output 1
       (type data)
       (name "Then")
       (lock name constraints)
      )
      (output 2
       (type data)
       (name "Else")
       (lock name constraints)
      )
     )
     (implementation
      (component isLocked no)
      (component expr "a!=0")
     )
     (views
      (icon
       (origin 355 310)
       (extent 46 25)
      )
      (detail
       (origin 209 308)
       (extent 73 64)
      )
      (terminals on)
      (active icon)
     )
    )
    (component 1 "TOGGLE"
     (name "Info About")
     (interface
      (sequence in)
      (sequence out)
      (input 1
       (type control)
       (name "Reset")
       (lock name constraints)
       (optional yes)
      )
      (output 1
       (type data)
       (lock name constraints)
      )
     )
     (implementation
      (component value Int32
       (name "Info About")
       (values range 0 1)
       (datum 0)
      )
      (component autoTrigger on)
      (component initializeAtPrerun off)
      (component initializeAtActivate off)
      (component initValue Int32
       (name "LongInt")
       (datum 0)
      )
     )
     (views
      (icon
       (origin 191 303)
       (extent 114 39)
       (format button)
      )
      (detail
       (origin 320 203)
       (extent 42 54)
      )
      (terminals on)
      (active icon)
     )
    )
    (component 2 "REPEATUNTILBREAK"
     (interface
      (sequence in)
      (sequence out)
      (output 1
       (type data)
       (name "Continuous")
       (lock name constraints)
      )
     )
     (implementation
     )
     (views
      (icon
       (origin 105 228)
       (extent 110 64)
       (iconImage "loop.icon")
      )
      (detail
      )
      (terminals on)
      (active icon)
     )
    )
    (component 3 "CONTEXT"
     (name "Info About...")
     (interface
      (sequence in)
      (sequence out)
     )
     (implementation
      (locked no)
      (trigMode deg)
      (nextID 2)
      (component 0 "NOTE"
       (name "NotePad")
       (interface
       )
       (implementation
        (component text 43 9
         PURPOSE:  Illustrate how one thread writes to a file and a second thread reads the
                   data and processes it at it's own speed.
         
         USER INPUT REQUIRED:  Press RUN and observe that the generating thread writes data faster
                   than the consuming thread can process.
         
         EXPLANATION:  It may be useful in some situations to create and store data in a file
                   so that it can be read back in and processed at non-real time rates.  This
                   example shows how one "thread" can create and write data to a file as quickly
                   as possible.  The second thread, running in parallel, reads the data back in
                   and performs some simulated, time-consuming processing on the data.
         
                   One important concept illustrated is the existence of independent "read"
                   and "write" file pointers for each file.  This allows independent reading
                   or writing in conjunction with Rewinding, etc.  Remember, however, that only
                   one read pointer exists for all "From File" objects with a common file name,
                   and only one write pointer exists for all "To File" objects with a common name.
         
                   It may be instructional to use the Edit Menu to enable "Show Execution FLow"
                   as you run this example; the "parallel" execution of each thread will be seen
                   clearly.
         
                   One final point:  Since the order of execution is not guaranteed, it is 
                   possible for the "consumer" thread to execute before the "producer" thread.
                   This might give a false EOF if there is not yet data in the file, or, if the
                   file doesn't exist, an Error would be generated by the Read.  
         
                   To solve this, we assume a new file is used each time the example is run; in
                   the example, we remove the file after all data is processed, so the same file
                   name can be reused.  An alternative would be to use a filename based on the
                   date and time.  In either case, the consuming thread must first loop repeatedly,
                   issuing a REWIND on the file name.  This will fail with an error until the file
                   actually exits.  You could also count the number of failures and exit if some
                   limit is reached.
         
                   After it is determined that the file exists, we must also make sure there is at
                   least one complete data record available for reading.  The same endless loop 
                   reads a record into the dummy variable "null";  if an EOF results, then not
                   enough data is yet available in the file.  Once the read succeeds, we REWIND the
                   file so we can terminate the first Until Break and begin processing data records
                   in the second Until Break.
         
                   After all data is processed, the file is removed using an Execute Program object.
        )
       )
       (views
        (icon
         (origin 391 338)
         (extent 78 25)
        )
        (detail
         (origin 16 192)
         (extent 848 437)
         (editing disabled)
        )
        (active detail)
       )
      )
      (component 1 "CONFIRM"
       (name "DONE")
       (interface
        (sequence in)
        (sequence out)
        (output 1
         (type data)
         (name "Go")
         (lock name constraints)
        )
       )
       (implementation
       )
       (views
        (icon
         (origin 410 459)
         (extent 58 37)
        )
        (detail
         (origin 466 633)
         (extent 79 34)
        )
        (terminals on)
        (active icon)
       )
      )
      (configuration
      )
      (ShowOnExecPanel
       (origin 55 78)
       (extent 866 560)
       (panel
        (origin 29 164)
        (extent 862 532)
        (widget 1 icon
         (title off)
         (origin 428 649)
         (extent 58 37)
        )
        (widget 0 detail
         (title off)
         (origin 34 189)
         (extent 848 437)
         (editing disabled)
        )
       )
      )
     )
     (views
      (icon
       (origin 397 418)
       (extent 127 25)
      )
      (detail
       (origin 19 164)
       (extent 862 513)
       (configuration
       )
       (stackingOrder 1 0)
      )
      (active icon)
     )
    )
    (configuration
     (connect D1:1 D0:1)
     (connect D2:1 D1:0)
     (connect D3:0 D1:1)
     (connect D0:1 D3:0)
    )
   )
   (views
    (icon
     (origin 465 628)
     (extent 110 25)
    )
    (panel
     (origin 32 103)
     (extent 297 74)
     (widget 1 icon
      (title off)
      (origin 134 116)
      (extent 114 39)
      (format button)
     )
    )
    (detail
     (origin 4 115)
     (extent 912 591)
     (configuration
      (connect D1:1 D0:1
       (Point 307 322)
       (Point 352 322)
      )
      (connect D2:1 D1:0
       (Point 217 260)
       (Point 248 260)
       (Point 248 300)
      )
      (connect D3:0 D1:1
       (Point 460 445)
       (Point 188 322)
      )
      (connect D0:1 D3:0
       (Point 403 312)
       (Point 410 312)
       (Point 410 120)
       (Point 460 120)
       (Point 460 415)
      )
     )
     (stackingOrder 0 1 2 3)
    )
    (active panel)
    (active panel)
   )
  )
  (component 14 "YPLOT"
   (name "Processed Data")
   (interface
    (sequence in)
    (sequence out)
    (input 1
     (type data)
     (name "Trace1")
     (lock constraints)
    )
   )
   (implementation
    (component clearAtPrerun 1)
    (component clearAtActivate 1)
    (component tracePin 1)
    (component graphMode strip)
   )
   (views
    (icon
     (extent 136 0)
     (iconImage "display.icon")
    )
    (detail
     (origin 705 493)
     (extent 271 174)
     (displayMode graphOnly)
     (graphType cartesian)
     (gridType off)
     (scale 0
      (name "Y name")
      (domainName "X name")
      (pen 9)
      (show 1)
      (range -1 2 3 linear)
     )
     (domain 0 6000 4 linear)
     (trace 0 onScale 0
      (name "Trace1")
      (pen 7)
      (lineType 1)
      (pointType 0)
     )
     (stepSize 1)
     (trackingEnabled 1)
    )
    (active detail)
   )
  )
  (component 15 "FORMULA"
   (name "linRamp(256,0,1)")
   (description 2 4
    Generate some output data
    
   )
   (interface
    (sequence in)
    (sequence out)
    (output 1
     (type data)
     (name "Result")
     (lock name constraints)
    )
   )
   (implementation
    (component isLocked no)
    (component expr 1 "ramp(256,0,1)")
   )
   (views
    (icon
     (origin 181 298)
     (extent 154 64)
     (iconImage "formula.icon")
    )
    (detail
     (origin 60 351)
     (extent 181 39)
    )
    (terminals on)
    (active detail)
   )
  )
  (component 16 "FORCOUNT"
   (interface
    (sequence in)
    (sequence out)
    (output 1
     (type data)
     (lock name constraints)
    )
   )
   (implementation
    (component count 20)
   )
   (views
    (icon
     (origin 45 238)
     (extent 110 64)
     (iconImage "loop.icon")
    )
    (detail
     (origin 20 250)
     (extent 141 41)
    )
    (active icon)
   )
  )
  (component 17 "SHELLESCAPE"
   (name "rm /tmp/example.dat")
   (description 8 4
    To insure that someone has not left the data file
    around that is owned by them, we remove it.  Since
    the /tmp dir is writable by all, we are allowed
    to remove the file even if we don't have write
    permission to change it.
    
    In a real application, this would have to be made
    more tolerant of multiple, simultaneous users.
   )
   (interface
    (sequence in)
    (sequence out)
    (output 1
     (type data)
     (name "Exit code")
     (lock name constraints)
    )
   )
   (implementation
    (attr iopath pipe read "pipe"
     (readTerm "\n")
     (fs " ")
     (eol "\n")
     (multiField fullSyntax)
     (arrayFormat block)
    )
    (attr iopath pipe write "pipe"
     (readTerm "\n")
     (fs " ")
     (eol "\n")
     (multiField fullSyntax)
     (arrayFormat block)
    )
    (component shell "sh")
    (component command "rm -f /tmp/example.data")
    (component deathWait "Yes")
   )
   (views
    (icon
     (origin 10 198)
     (extent 181 25)
    )
    (detail
     (origin 6 204)
     (extent 389 53)
    )
    (active icon)
   )
  )
  (component 18 "COUNTER"
   (name "# Records Written")
   (interface
    (sequence in)
    (sequence out)
    (input 1
     (type data)
     (name "Data")
    )
    (output 1
     (type data)
     (name "Count")
     (tag "Count")
     (lock name constraints)
    )
   )
   (implementation
    (component clearAtPrerun 1)
    (component clearAtActivate 1)
   )
   (views
    (icon
     (extent 163 0)
    )
    (detail
     (origin 380 357)
     (extent 241 26)
    )
    (active detail)
   )
  )
  (component 19 "TOFILE"
   (interface
    (sequence in)
    (sequence out)
    (input 1
     (type data)
     (name "A")
     (optional yes)
    )
   )
   (implementation
    (attr iopath file write "/tmp/example.data"
     (readTerm "\n")
     (fs " ")
     (eol "\n")
     (multiField fullSyntax)
     (arrayFormat block)
    )
    (procedure
      WRITE TEXT a EOL
    )
   )
   (views
    (icon
     (origin 521 348)
     (extent 78 25)
    )
    (detail
     (origin 392 226)
     (extent 267 89)
    )
    (terminals on)
    (active detail)
   )
  )
  (component 20 "YPLOT"
   (name "Generated Data")
   (interface
    (sequence in)
    (sequence out)
    (input 1
     (type data)
     (name "Trace1")
     (lock constraints)
    )
   )
   (implementation
    (component clearAtPrerun 1)
    (component clearAtActivate 1)
    (component tracePin 1)
    (component graphMode strip)
   )
   (views
    (icon
     (extent 136 0)
     (iconImage "display.icon")
    )
    (detail
     (origin 703 262)
     (extent 274 177)
     (displayMode graphOnly)
     (graphType cartesian)
     (gridType off)
     (scale 0
      (name "Y name")
      (domainName "X name")
      (pen 9)
      (show 1)
      (range -1 2 3 linear)
     )
     (domain 0 6000 4 linear)
     (trace 0 onScale 0
      (name "Trace1")
      (pen 4)
      (lineType 1)
      (pointType 0)
     )
     (stepSize 1)
     (trackingEnabled 1)
    )
    (active detail)
   )
  )
  (configuration
   (connect D12:0 D2:0)
   (connect D12:1 D3:1)
   (connect D12:2 D4:0)
   (connect D4:0 D5:0)
   (connect D9:0 D6:0)
   (connect D9:1 D7:0)
   (connect D9:2 D8:0)
   (connect D10:1 D9:0)
   (connect D10:0 D11:0)
   (connect D11:1 D12:0)
   (connect D12:1 D14:1)
   (connect D16:1 D15:0)
   (connect D17:0 D16:0)
   (connect D15:1 D18:1)
   (connect D15:1 D19:1)
   (connect D15:1 D20:1)
  )
 )
 (views
  (detail
   (origin 0 62)
   (extent 980 643)
   (configuration
    (connect D12:0 D2:0
     (Point 190 789)
     (Point 190 805)
    )
    (connect D12:1 D3:1
     (Point 366 730)
     (Point 390 730)
     (Point 390 670)
     (Point 406 670)
    )
    (connect D12:2 D4:0
     (Point 366 770)
     (Point 400 770)
     (Point 400 720)
     (Point 550 720)
     (Point 550 735)
    )
    (connect D4:0 D5:0
     (Point 550 765)
     (Point 550 785)
    )
    (connect D9:0 D6:0
     (Point 360 605)
     (Point 360 620)
     (Point 340 620)
     (Point 340 625)
    )
    (connect D9:1 D7:0
     (Point 565 520)
     (Point 610 520)
     (Point 610 525)
    )
    (connect D9:2 D8:0
     (Point 565 580)
     (Point 610 580)
     (Point 610 585)
    )
    (connect D10:1 D9:0
     (Point 137 450)
     (Point 360 450)
     (Point 360 467)
    )
    (connect D10:0 D11:0
     (Point 80 484)
     (Point 80 595)
    )
    (connect D11:1 D12:0
     (Point 137 630)
     (Point 190 630)
     (Point 190 683)
    )
    (connect D12:1 D14:1
     (Point 366 730)
     (Point 390 730)
     (Point 390 710)
     (Point 670 710)
     (Point 670 580)
     (Point 702 580)
    )
    (connect D16:1 D15:0
     (Point 157 270)
     (Point 170 270)
     (Point 170 320)
    )
    (connect D17:0 D16:0
     (Point 100 225)
     (Point 100 235)
    )
    (connect D15:1 D18:1
     (Point 313 370)
     (Point 377 370)
    )
    (connect D15:1 D19:1
     (Point 313 370)
     (Point 330 370)
     (Point 330 270)
     (Point 349 270)
    )
    (connect D15:1 D20:1
     (Point 313 370)
     (Point 360 370)
     (Point 360 410)
     (Point 640 410)
     (Point 640 350)
     (Point 700 350)
    )
   )
   (stackingOrder 0 1 2 3 4 5 6 7 8 9
     10 11 12 14 15 16 17 18 19 20
     13)
  )
  (active detail)
  (numberFormats
   (realFormat standard)
   (realSigDigits 4)
   (realRadixSpec 4)
   (integerBase decimal)
  )
  (waveformSettings
   (timeSpan 0.02)
   (numPoints 256)
  )
 )
)
